# UESLock is a helper class for secure lock handling. Usage pattern:
#
# lock = UESLock.create(resourceUri, createDto)
# begin
#   # ...
#   # a business logic here
#   # ...
# ensure
#   lock.unlock
# end
#
# <tt><b>since</b></tt> UES7-10
class Ues::Platform::Concurrent::UESLock
  private_class_method :new

  @uesLock = nil
  # :stopdoc:
  def initialize(lock)
    @uesLock = lock
  end
  # :startdoc:

  # Creates new lock.
  #
  # <tt><b>param</b></tt> resourceUri UESURI of resource to be locked
  #
  # <tt><b>param</b></tt> lock Additional lock attributes
  #
  # <tt><b>return</b></tt> Instance of lock or null if lock was not create (e.g. resource is already locked)
  #
  # <tt><b>since</b></tt> UES7-10
  def self.create(resourceUri, lock=nil)
    resourceUri = Ues::Platform::Commons::Uri::UESURI.new(resourceUri).to_java
    if (lock != nil)
      lock = Ues::Platform::Concurrent::UESLockCreate.new(lock).to_java
      _r = Java::cz.ues.platform.concurrent.UESLock.create(resourceUri, lock)
    else
      _r = Java::cz.ues.platform.concurrent.UESLock.create(resourceUri, nil)
    end
    if (_r != nil)
      return UESLock.send("new", _r)
    else
      return nil
    end
  end

  # Checks if lock is valid (e.g. is not expired or was not removed by other process).
  #
  # <tt><b>return</b></tt> True if lock is valid, else false
  #
  # <tt><b>since</b></tt> UES7-10
  def isValid()
    return @uesLock.isValid()
  end
  alias is_valid isValid
  alias valid? isValid

  # Reallocates lock and thus allows prolonging (or shortening)
  # its expiration time. In case lock was already removed (e.g.
  # is already expired, or was removed by other process), exception
  # is thrown.
  #
  # <tt><b>param</b></tt> lockTimeout New value of lock timeout in seconds (may be null or -1 to maintain lock timeout initially set on lock create)
  #
  # <tt><b>since</b></tt> UES7-10
  def reallocate(lockTimeout=nil)
    @uesLock.reallocate(lockTimeout)
  end

  # Unlocks lock. In case lock is not valid in time of its
  # unlock exception is thrown.
  #
  # <tt><b>since</b></tt> UES7-10
  def unlock()
    @uesLock.unlock()
  end

end
